searchNodes=[{"doc":"Provides a standard API to handle keyword-list-based options.NimbleOptions allows developers to create schemas using a pre-defined set of options and types. The main benefits are:A single unified way to define simple static optionsConfig validation against schemasAutomatic doc generationSchema optionsThese are the options supported in a schema. They are what defines the validation for the items in the given schema.:type - The type of the option item. The default value is :any.:required - Defines if the option item is required. The default value is false.:default - The default value for option item if not specified.:keys - Available for types :keyword_list and :non_empty_keyword_list, it defines which set of keys are accepted for the option item. The value of the :keys option is a schema itself. For example: keys: [foo: [type: :atom]]. Use :* as the key to allow multiple arbitrary keys and specify their schema: keys: [*: [type: :integer]].:deprecated - Defines a message to indicate that the option item is deprecated. The message will be displayed as a warning when passing the item.:rename_to - Renames a option item allowing one to use a normalized name internally, e.g. rename a deprecated item to the currently accepted name.:doc - The documentation for the option item.:subsection - The title of separate subsection of the options' documentationTypes:any - Any type.:keyword_list - A keyword list.:non_empty_keyword_list - A non-empty keyword list.:atom - An atom.:string - A string.:boolean - A boolean.:integer - An integer.:non_neg_integer - A non-negative integer.:pos_integer - A positive integer.:timeout - A non-negative integer or the atom :infinity.:pid - A PID (process identifier).:mfa - A named function in the format {module, function, arity}:mod_arg - A module along with arguments, e.g. {MyModule, [arg1, arg2]}. Usually used for process initialization using start_link and friends.{:fun, arity} - Any function with the specified arity.{:in, choices} - A value that is a member of one of the choices. choices should be a list of terms. The value is an element in said list of terms, that is, value in choices is true. Previously called :one_of.{:custom, mod, fun, args} - A custom type. The related value must be validated by mod.fun(values, ...args). The function should return {:ok, value} or {:error, message}.{:or, subtypes} - A value that matches one of the given subtypes. The value is matched against the subtypes in the order specified in the list of subtypes. If one of the subtypes matches and updates (casts) the given value, the updated value is used. For example: {:or, [:string, :boolean, {:fun, 2}]}. If one of the subtypes is a keyword list, you won't be able to pass :keys directly. For this reason, keyword lists (:keyword_list and :non_empty_keyword_list) are special cased and can be used as subtypes with {:keyword_list, keys} or {:non_empty_keyword_list, keys}. For example, a type such as {:or, [:boolean, keyword_list: [enabled: [type: :boolean]]]} would match either a boolean or a keyword list with the :enabled boolean option in it.{:list, subtype} - A list where all elements match subtype. subtype can be any of the accepted types listed here. Empty lists are allowed. The resulting validated list contains the validated (and possibly updated) elements, each as returned after validation through subtype. For example, if subtype is a custom validator function that returns an updated value, then that updated value is used in the resulting list. Validation fails at the first element that is invalid according to subtype.Exampleiex&gt; schema = [ ...&gt; producer: [ ...&gt; type: :non_empty_keyword_list, ...&gt; required: true, ...&gt; keys: [ ...&gt; module: [required: true, type: :mod_arg], ...&gt; concurrency: [ ...&gt; type: :pos_integer, ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; config = [ ...&gt; producer: [ ...&gt; concurrency: 1, ...&gt; ] ...&gt; ] ...&gt; ...&gt; {:error, %NimbleOptions.ValidationError{} = error} = NimbleOptions.validate(config, schema) ...&gt; Exception.message(error) &quot;required option :module not found, received options: [:concurrency] (in options [:producer])&quot;Nested option itemsNimbleOptions allows option items to be nested so you can recursively validate any item down the options tree.Exampleiex&gt; schema = [ ...&gt; producer: [ ...&gt; required: true, ...&gt; type: :non_empty_keyword_list, ...&gt; keys: [ ...&gt; rate_limiting: [ ...&gt; type: :non_empty_keyword_list, ...&gt; keys: [ ...&gt; interval: [required: true, type: :pos_integer] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; config = [ ...&gt; producer: [ ...&gt; rate_limiting: [ ...&gt; interval: :oops! ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; {:error, %NimbleOptions.ValidationError{} = error} = NimbleOptions.validate(config, schema) ...&gt; Exception.message(error) &quot;expected :interval to be a positive integer, got: :oops! (in options [:producer, :rate_limiting])&quot;","ref":"NimbleOptions.html","title":"NimbleOptions","type":"module"},{"doc":"Returns documentation for the given schema.You can use this to inject documentation in your docstrings. For example, say you have your schema in a module attribute:@options_schema [...]With this, you can use docs/1 to inject documentation:@doc &quot;Supported options:\\n\#{NimbleOptions.docs(@options_schema)}&quot;Options:nest_level - an integer deciding the &quot;nest level&quot; of the generated docs. This is useful when, for example, you use docs/2 inside the :doc option of another schema. For example, if you have the following nested schema:nested_schema = [ allowed_messages: [type: :pos_integer, doc: &quot;Allowed messages.&quot;], interval: [type: :pos_integer, doc: &quot;Interval.&quot;] ]then you can document it inside another schema with its nesting level increased:schema = [ producer: [ type: {:or, [:string, keyword_list: nested_schema]}, doc: &quot;Either a string or a keyword list with the following keys:\\n\\n&quot; &lt;&gt; NimbleOptions.docs(nested_schema, nest_level: 1) ], other_key: [type: :string] ]","ref":"NimbleOptions.html#docs/2","title":"NimbleOptions.docs/2","type":"function"},{"doc":"Validate the given options with the given schema.See the module documentation for what a schema is.If the validation is successful, this function returns {:ok, validated_options} where validated_options is a keyword list. If the validation fails, this function returns {:error, validation_error} where validation_error is a NimbleOptions.ValidationError struct explaining what's wrong with the options. You can use raise/1 with that struct or Exception.message/1 to turn it into a string.","ref":"NimbleOptions.html#validate/2","title":"NimbleOptions.validate/2","type":"function"},{"doc":"Validates the given options with the given schema and raises if they're not valid.This function behaves exactly like validate/2, but returns the options directly if they're valid or raises a NimbleOptions.ValidationError exception otherwise.","ref":"NimbleOptions.html#validate!/2","title":"NimbleOptions.validate!/2","type":"function"},{"doc":"A schema. See the module documentation for more information.","ref":"NimbleOptions.html#t:schema/0","title":"NimbleOptions.schema/0","type":"type"},{"doc":"An error that is returned (or raised) when options are invalid.Only the following documented fields are considered public. All other fields are considered private and should not be referenced::key - The key that did not successfully validate.:keys_path - If the key is nested, this is the path to the key.:value - The value that failed to validate. This field is nil if there was no value provided.Since this is an exception, you can either raise it directly with raise/1 or turn it into a message string with Exception.message/1.","ref":"NimbleOptions.ValidationError.html","title":"NimbleOptions.ValidationError","type":"exception"},{"doc":"","ref":"NimbleOptions.ValidationError.html#t:t/0","title":"NimbleOptions.ValidationError.t/0","type":"type"}]